library wood_keith_djniexample_Example1;

{
  Pascal skeleton for using JNI with the wood.keith.djniexample.Example1 Java class.

  Generated Thu Apr 03 22:44:28 GMT+10:00 2003 by JavaD.
  Updated by Keith Wood (kbwood@iprimus.com.au).
}

uses
  SysUtils, JNI,
{$IFDEF JNIUTILS}
  JNIUtils,
{$ENDIF}
  DB, DBTables;

{
  Java declaration: public static native void printMessage(java.lang.String)
  Class: wood.keith.djniexample.Example1
  Method: printMessage
  Signature: (Ljava/lang/String;)V

  Display the message provided.
}
procedure Java_wood_keith_djniexample_Example1_printMessage(
  PEnv: PJNIEnv; Obj: JObject; Message: JString);
  {$IFDEF WIN32} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
var
  JVM: TJNIEnv;
begin
  JVM := TJNIEnv.Create(PEnv);
  try
    Writeln(JVM.JStringToString(Message) + ' at ' + DateTimeToStr(Now));
  finally
    JVM.Free;
  end;
end;

{
  Search BioLife database, given a query, and return matches.
}
function searchBioLife(JVM: TJNIEnv; QueryWhere: string): JObjectArray;
var
  BioLifeQuery: TQuery;
  ExceptionClass: JObject;
  BioLifeClass: JObject;
{$IFNDEF JNIUTILS}
  ConstructorId: JMethodID;
{$ENDIF}
  BioLifeObj: JObject;
  Index: Integer;
begin
  Result       := nil;
  BioLifeQuery := TQuery.Create(nil);
  try
    // Find exception classes
    ExceptionClass := JVM.FindClass('java/lang/Exception');
    if not Assigned(ExceptionClass) then
      Writeln('Cannot find Exception class');

    // Find BioLife value object class
    BioLifeClass := JVM.FindClass('wood/keith/djniexample/BioLife');
    if not Assigned(BioLifeClass) then
      Exit;
{$IFNDEF JNIUTILS}
    // And its constructor
    ConstructorId := JVM.GetMethodID(BioLifeClass, '<init>',
      '(ILjava/lang/String;Ljava/lang/String;Ljava/lang/String;ID)V');
    if not Assigned(ConstructorId) then
      Exit;
{$ENDIF}

    // Run the query
    with BioLifeQuery do
    begin
      DatabaseName := 'DBDemos';
      SQL.Text     := 'select BioLife."Species No", Category, Common_Name, ' +
        'BioLife."Species Name", BioLife."Length (cm)", Length_In ' +
        'from BioLife where ' + QueryWhere;
      Open;
      if RecordCount = 0 then
      begin
        JVM.ThrowNew(ExceptionClass, 'No matching fish found');
        Exit;
      end;

      // Create the result array
      Result := JVM.NewObjectArray(RecordCount, BioLifeClass, nil);
      Index  := 0;
      while not EOF do
      begin
        // Create a BioLife object and populate it
{$IFDEF JNIUTILS}
        BioLifeObj := JNIUtils.CreateObject(JVM,
          'wood.keith.djniexample.BioLife',
          'void (int, String, String, String, int, double)',
          [FieldByName('Species No').AsInteger,
          FieldByName('Category').AsString,
          FieldByName('Common_Name').AsString,
          FieldByName('Species Name').AsString,
          FieldByName('Length (cm)').AsInteger,
          FieldByName('Length_In').AsFloat]);
{$ELSE}
        BioLifeObj := JVM.NewObjectA(BioLifeClass, ConstructorId,
          JVM.ArgsToJValues([FieldByName('Species No').AsInteger,
          FieldByName('Category').AsString,
          FieldByName('Common_Name').AsString,
          FieldByName('Species Name').AsString,
          FieldByName('Length (cm)').AsInteger,
          FieldByName('Length_In').AsFloat]));
{$ENDIF}
        if not Assigned(BioLifeClass) then
          Exit;
        // Add it to the array
        JVM.SetObjectArrayElement(Result, Index, BioLifeObj);
        Inc(Index);
        Next;
      end;
      Close;
    end;
  finally
    BioLifeQuery.Free;
  end;
end;

{
  Java declaration: public native wood.keith.djniexample.BioLife[] getBioLife()
  Class: wood.keith.djniexample.Example1
  Method: getBioLife
  Signature: ()[Lwood/keith/djniexample/BioLife;

  Find all BioLife records.
}
function Java_wood_keith_djniexample_Example1_getBioLife__(
  PEnv: PJNIEnv; Obj: JObject; Category: JString): JObjectArray;
  {$IFDEF WIN32} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
var
  JVM: TJNIEnv;
begin
  JVM := TJNIEnv.Create(PEnv);
  try
    Result := searchBioLife(JVM, 'BioLife."Length (cm)" > 0');
  finally
    JVM.Free;
  end;
end;

{
  Java declaration: public native wood.keith.djniexample.BioLife[] getBioLife(java.lang.String)
  Class: wood.keith.djniexample.Example1
  Method: getBioLife
  Signature: (Ljava/lang/String;)[Lwood/keith/djniexample/BioLife;

  Find BioLife records like a given category.
}
function Java_wood_keith_djniexample_Example1_getBioLife__Ljava_lang_String_2(
  PEnv: PJNIEnv; Obj: JObject; Category: JString): JObjectArray;
  {$IFDEF WIN32} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
var
  JVM: TJNIEnv;
begin
  JVM := TJNIEnv.Create(PEnv);
  try
    Result := searchBioLife(JVM,
      'Category like "%' + JVM.JStringToString(Category) + '%"');
  finally
    JVM.Free;
  end;
end;

{
  Java declaration: public native wood.keith.djniexample.BioLife[] getBioLife(int, int)
  Class: wood.keith.djniexample.Example1
  Method: getBioLife
  Signature: (II)[Lwood/keith/djniexample/BioLife;

  Find BioLife records with length (cm) in a given range.
}
function Java_wood_keith_djniexample_Example1_getBioLife__II(
  PEnv: PJNIEnv; Obj: JObject; MinLength: JInt; MaxLength: JInt): JObjectArray;
  {$IFDEF WIN32} stdcall; {$ENDIF} {$IFDEF LINUX} cdecl; {$ENDIF}
var
  JVM: TJNIEnv;
begin
  JVM := TJNIEnv.Create(PEnv);
  try
    Result := searchBioLife(JVM, 'BioLife."Length (cm)" between ' +
      IntToStr(MinLength) + ' and ' + IntToStr(MaxLength));
  finally
    JVM.Free;
  end;
end;

exports

  // Make the routines available for external use

  Java_wood_keith_djniexample_Example1_printMessage,
  Java_wood_keith_djniexample_Example1_getBioLife__,
  Java_wood_keith_djniexample_Example1_getBioLife__Ljava_lang_String_2,
  Java_wood_keith_djniexample_Example1_getBioLife__II;
end.
