package wood.keith.djniexample;

/**
 * Demonstrate JNI access to Delphi library.
 *
 * @author   Keith Wood (kbwood@iprimus.com.au)
 * @version  1.0  3 April 2003
 */
public class Example1 {

  static {
    System.out.println("Library path: " + System.getProperty("java.library.path"));
    // Load the native library
    System.loadLibrary("wood_keith_djniexample_Example1");
  }

  public Example1() {}

  /**
   * Display the message to the console.
   *
   * @param  message  the text to display
   */
  public static native void printMessage(String message);

  /**
   * Find all BioLife database records.
   *
   * @return  an array of matching BioLife records, empty if none
   */
  public native BioLife[] getBioLife();

  /**
   * Find BioLife database records matching the given category.
   *
   * @param  category  the (partial) category to match on
   * @return  an array of matching BioLife records, empty if none
   */
  public native BioLife[] getBioLife(String category);

  /**
   * Find BioLife database records matching the given lengths.
   *
   * @param  minLength  the minimum length required
   * @param  maxLength  the maximum length required
   * @return  an array of matching BioLife records, empty if none
   */
  public native BioLife[] getBioLife(int minLength, int maxLength);

  /**
   * Search for matching fish.
   */
  private void performSearch() {
    System.out.println("\nSearching fish for everything");
    try {
      BioLife[] fish = getBioLife();
      for (int index = 0; index < fish.length; index++) {
        System.out.println("Found " + fish[index]);
      }
    }
    catch (Exception e) {
      System.out.println("Found none");
      e.printStackTrace();
    }
  }

  /**
   * Search for matching fish.
   *
   * @param  category  the partial category to look for
   */
  private void performSearch(String category) {
    System.out.println("\nSearching fish for \"" + category + "\"");
    try {
      BioLife[] fish = getBioLife(category);
      for (int index = 0; index < fish.length; index++) {
        System.out.println("Found " + fish[index]);
      }
    }
    catch (Exception e) {
      System.out.println("Found none");
      e.printStackTrace();
    }
  }

  /**
   * Search for matching fish.
   *
   * @param  minLength  the minimum length required
   * @param  maxLength  the maximum length required
   */
  private void performSearch(int minLength, int maxLength) {
    System.out.println("\nSearching fish between " + minLength +
      " and " + maxLength + "cm");
    try {
      BioLife[] fish = getBioLife(minLength, maxLength);
      for (int index = 0; index < fish.length; index++) {
        System.out.println("Found " + fish[index]);
      }
    }
    catch (Exception e) {
      System.out.println("Found none");
      e.printStackTrace();
    }
  }

  /**
   * Demonstrate calling native methods.
   *
   * @param  args  command-line parameters
   */
  public static void main(String[] args) {
    Example1 example1 = new Example1();
    printMessage("Starting example1");
    example1.performSearch();
    example1.performSearch("Ray");
    example1.performSearch("XRay");
    example1.performSearch(20, 40);
    printMessage("Ending example1");
  }
}

/**
 * Value object for BioLife records.
 */
class BioLife {

  private int _speciesNo;
  private String _category;
  private String _commonName;
  private String _speciesName;
  private int _lengthCM;
  private double _lengthIN;

  public BioLife(int speciesNo, String category, String commonName,
      String speciesName, int lengthCM, double lengthIN) {
    _speciesNo = speciesNo;
    _category = category;
    _commonName = commonName;
    _speciesName = speciesName;
    _lengthCM = lengthCM;
    _lengthIN = lengthIN;
  }

  public int getSpeciesNo() { return _speciesNo; }
  public String getCategory() { return _category; }
  public String getCommonName() { return _commonName; }
  public String getSpeciesName() { return _speciesName; }
  public int getLengthCM() { return _lengthCM; }
  public double getLengthIN() { return _lengthIN; }

  public String toString() {
    return _commonName + " (" + _speciesName + ") - length " +
      _lengthCM + "cm (" + (Math.round(_lengthIN * 100) / 100.0) + "in)";
  }
}