package wood.keith.opentools.javad;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JTextField;

import com.borland.jbuilder.ide.PackageBrowserTree;
import com.borland.jbuilder.node.JBProject;
import com.borland.primetime.node.Project;
import com.borland.primetime.util.VetoException;
import com.borland.primetime.wizard.BasicWizardPage;

/**
 * This class contains wizard page 1 of the JavaD Wizard.
 *
 * @author   Keith Wood (kbwood@iprimus.com.au)
 * @version  1.0  16 March 2003
 */
public class JavaDWizardPage1 extends BasicWizardPage {

  private Project project;

  private GridBagLayout gridBagLayout = new GridBagLayout();
  private JLabel jLabel1 = new JLabel();
  private JTextField classNameText = new JTextField();
  private JButton classNameButton = new JButton();
  private JLabel jLabel2 = new JLabel();
  private JTextField directoryText = new JTextField();
  private JButton directoryButton = new JButton();
  JCheckBox openCheck = new JCheckBox();

  /**
   * Create a new wizard page.
   *
   * @param  project  the current project
   */
  public JavaDWizardPage1(Project project) {
    this.project = project;
    try {
      jbInit();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

  /**
   * Define the user interface.
   * The user interface is constructed here so the JBuilder UI Designer
   * may be used to edit it.
   */
  protected void jbInit() throws Exception {
    this.setPageTitle("Generate Pascal JNI Skeleton");
    this.setLayout(gridBagLayout);
    try {
      this.setLargeIcon(new ImageIcon(ClassLoader.getSystemResource(
        "com/borland/jbuilder/wizard/method/MethodWizardLarge.gif")));
    }
    catch (Exception e) {
      // Ignore
    }
    this.setInstructions(
      "Enter, or select, the name of the class to provide a Pascal JNI skeleton " +
      "for, along with the directory where the code should appear.");
    jLabel1.setDisplayedMnemonic('C');
    jLabel1.setLabelFor(classNameText);
    jLabel1.setText("Class name");
    classNameButton.setMinimumSize(new Dimension(57, 21));
    classNameButton.setPreferredSize(new Dimension(57, 21));
    classNameButton.setMargin(new Insets(2, 2, 2, 2));
    classNameButton.setMnemonic('B');
    classNameButton.setText("Browse");
    classNameButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        classNameButton_actionPerformed(e);
      }
    });
    jLabel2.setDisplayedMnemonic('D');
    jLabel2.setLabelFor(directoryText);
    jLabel2.setText("Output directory");
    directoryButton.setMinimumSize(new Dimension(57, 21));
    directoryButton.setPreferredSize(new Dimension(57, 21));
    directoryButton.setMargin(new Insets(2, 2, 2, 2));
    directoryButton.setMnemonic('R');
    directoryButton.setText("Browse");
    directoryButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        directoryButton_actionPerformed(e);
      }
    });
    directoryText.setText("");
    classNameText.setText("");
    openCheck.setMnemonic('O');
    openCheck.setText("Open Pascal file");
    openCheck.setSelected(true);
    this.add(jLabel1, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
        , GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(2, 2, 2, 2), 0, 0));
    this.add(classNameText, new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0
        , GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(2, 2, 2, 2), 0, 0));
    this.add(classNameButton, new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0
        , GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(2, 2, 2, 2), 0, 0));
    this.add(jLabel2, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
        , GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(2, 2, 2, 2), 0, 0));
    this.add(directoryText, new GridBagConstraints(1, 1, 1, 1, 1.0, 0.0
        , GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(2, 2, 2, 2), 0, 0));
    this.add(directoryButton, new GridBagConstraints(2, 1, 1, 1, 0.0, 0.0
        , GridBagConstraints.NORTH, GridBagConstraints.NONE, new Insets(2, 2, 2, 2), 0, 0));
    this.add(openCheck,  new GridBagConstraints(1, 2, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
  }

  /**
   * Browse for a class to review.
   *
   * @param  e  the triggering event
   */
  void classNameButton_actionPerformed(ActionEvent e) {
    String className = PackageBrowserTree.browseClass(
      (JBProject)project, this, "Select Native Class", classNameText.getText());
    if (className != null) {
      classNameText.setText(className);
    }
  }

  /**
   * Browse for an output directory.
   *
   * @param  e  the triggering event
   */
  void directoryButton_actionPerformed(ActionEvent e) {
    JFileChooser chooser = new JFileChooser(directoryText.getText());
    chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
    if (chooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
      directoryText.setText(chooser.getSelectedFile().getAbsolutePath());
    }
  }

  /**
   * Check for user input error.
   * The user has completed this page so check for input error.
   * If any, then report the error in a dialog and throw an exception.
   */
  public void checkPage() throws VetoException {
    checkCondition(getClassName().length() > 0,
      "Please enter the class name", classNameText);
    checkCondition(getDirectory().length() > 0,
      "Please enter the output directory", directoryText);
  }

  /**
   * Check for a user input error.
   *
   * @param  condition  a test that must be true for the page to be valid
   * @param  message    the text to display
   * @param  focus      the UI component to move focus to
   * @throws  VetoException  if the condition is not met
   */
  private void checkCondition(boolean condition, String message,
      JComponent focus) throws VetoException {
    if (!condition) {
      focus.requestFocus();
      JOptionPane.showMessageDialog(wizardHost.getDialogParent(), message,
        JavaDWizard.TITLE, JOptionPane.ERROR_MESSAGE, null);
      throw new VetoException();
    }
  }

  /**
   * Get the name of the class to examine for native methods.
   *
   * @return  the class name
   */
  public String getClassName() { return classNameText.getText(); }

  /**
   * Get the output directory.
   *
   * @return  the name of the directory
   */
  public String getDirectory() { return directoryText.getText(); }

  /**
   * Discover whether to open the generated file.
   *
   * @return  true if file is to be opened, false if not
   */
  public boolean isOpenFile() { return openCheck.isSelected(); }
}